@@ -28,21 +28,19 @@
 #include <cassert>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <algorithm>
 
 ViewBackend::ViewBackend(ClientBundle* clientBundle, struct wpe_view_backend* backend)
     : m_clientBundle(clientBundle)
     , m_backend(backend)
 {
     m_clientBundle->viewBackend = this;
-
-    wl_list_init(&m_frameCallbacks);
 }
 
 ViewBackend::~ViewBackend()
 {
-    clearFrameCallbacks();
-
-    unregisterSurface(m_surfaceId);
+    while (!m_bridgeIds.empty())
+        unregisterSurface(m_bridgeIds.front());
 
     if (m_clientFd != -1)
         close(m_clientFd);
@@ -74,15 +72,6 @@ int ViewBackend::clientFd()
     return dup(m_clientFd);
 }
 
-void ViewBackend::frameCallback(struct wl_resource* callback)
-{
-    auto* resource = new FrameCallbackResource;
-    resource->resource = callback;
-    resource->destroyListener.notify = FrameCallbackResource::destroyNotify;
-    wl_resource_add_destroy_listener(callback, &resource->destroyListener);
-    wl_list_insert(&m_frameCallbacks, &resource->link);
-}
-
 void ViewBackend::exportBufferResource(struct wl_resource* bufferResource)
 {
     m_clientBundle->exportBuffer(bufferResource);
@@ -105,44 +94,32 @@ void ViewBackend::exportEGLStreamProducer(struct wl_resource* bufferResource)
 
 void ViewBackend::dispatchFrameCallbacks()
 {
-    FrameCallbackResource* resource;
-    wl_list_for_each(resource, &m_frameCallbacks, link) {
-        wl_callback_send_done(resource->resource, 0);
+    if (G_LIKELY(!m_bridgeIds.empty())) {
+        WS::Instance::singleton().dispatchFrameCallbacks(m_bridgeIds.back());
+        wpe_view_backend_dispatch_frame_displayed(m_backend);
     }
-    clearFrameCallbacks();
-
-    if (m_client.object)
-        wl_client_flush(m_client.object);
-    wpe_view_backend_dispatch_frame_displayed(m_backend);
 }
 
 void ViewBackend::releaseBuffer(struct wl_resource* buffer_resource)
 {
     wl_buffer_send_release(buffer_resource);
-    if (m_client.object)
-        wl_client_flush(m_client.object);
+    wl_client_flush(wl_resource_get_client(buffer_resource));
 }
 
-void ViewBackend::registerSurface(uint32_t surfaceId)
+void ViewBackend::registerSurface(uint32_t bridgeId)
 {
-    m_surfaceId = surfaceId;
-    m_client.object = WS::Instance::singleton().registerViewBackend(m_surfaceId, *this);
-
-    m_client.destroyListener.notify = Client::destroyNotify;
-    wl_client_add_destroy_listener(m_client.object, &m_client.destroyListener);
+    m_bridgeIds.push_back(bridgeId);
+    WS::Instance::singleton().registerViewBackend(m_bridgeIds.back(), *this);
 }
 
-void ViewBackend::unregisterSurface(uint32_t surfaceId)
+void ViewBackend::unregisterSurface(uint32_t bridgeId)
 {
-    if (!surfaceId || m_surfaceId != surfaceId)
+    auto it = std::find(m_bridgeIds.begin(), m_bridgeIds.end(), bridgeId);
+    if (it == m_bridgeIds.end())
         return;
 
-    clearFrameCallbacks();
-
-    g_clear_pointer(&m_client.object, wl_client_destroy);
-
-    WS::Instance::singleton().unregisterViewBackend(m_surfaceId);
-    m_surfaceId = 0;
+    m_bridgeIds.erase(it);
+    WS::Instance::singleton().unregisterViewBackend(bridgeId);
 }
 
 void ViewBackend::didReceiveMessage(uint32_t messageId, uint32_t messageBody)
@@ -158,33 +135,3 @@ void ViewBackend::didReceiveMessage(uint32_t messageId, uint32_t messageBody)
         assert(!"WPE fdo received an invalid IPC message");
     }
 }
-
-void ViewBackend::clearFrameCallbacks()
-{
-    FrameCallbackResource* resource;
-    FrameCallbackResource* next;
-    wl_list_for_each_safe(resource, next, &m_frameCallbacks, link) {
-        wl_list_remove(&resource->link);
-        wl_list_remove(&resource->destroyListener.link);
-        wl_resource_destroy(resource->resource);
-        delete resource;
-    }
-    wl_list_init(&m_frameCallbacks);
-}
-
-void ViewBackend::Client::destroyNotify(struct wl_listener* listener, void*)
-{
-    Client* client;
-    client = wl_container_of(listener, client, destroyListener);
-
-    client->object = nullptr;
-}
-
-void ViewBackend::FrameCallbackResource::destroyNotify(struct wl_listener* listener, void*)
-{
-    FrameCallbackResource* resource;
-    resource = wl_container_of(listener, resource, destroyListener);
-
-    wl_list_remove(&resource->link);
-    delete resource;
-}
