@@ -31,6 +31,7 @@
 
 #include <gio/gio.h>
 #include <wpe/wpe.h>
+#include <vector>
 
 class ViewBackend;
 
@@ -64,46 +65,32 @@ public:
 
     void initialize();
     int clientFd();
-    void frameCallback(struct wl_resource* callbackResource) override;
     void exportBufferResource(struct wl_resource* bufferResource) override;
     void exportLinuxDmabuf(const struct linux_dmabuf_buffer *dmabuf_buffer) override;
     void exportShmBuffer(struct wl_resource* bufferResource, struct wl_shm_buffer* shmBuffer) override;
     void exportEGLStreamProducer(struct wl_resource*) override;
+
+    void bridgeConnectionLost(uint32_t id) override
+    {
+         unregisterSurface(id);
+    }
+
     void dispatchFrameCallbacks();
     void releaseBuffer(struct wl_resource* buffer_resource);
 
 private:
-    struct FrameCallbackResource {
-        struct wl_resource* resource;
-
-        struct wl_list link;
-        struct wl_listener destroyListener;
-
-        static void destroyNotify(struct wl_listener*, void*);
-    };
-
     void didReceiveMessage(uint32_t messageId, uint32_t messageBody) override;
 
     void registerSurface(uint32_t);
     void unregisterSurface(uint32_t);
 
-    void clearFrameCallbacks();
-
     static gboolean s_socketCallback(GSocket*, GIOCondition, gpointer);
 
-    uint32_t m_surfaceId { 0 };
-    struct Client {
-        struct wl_client* object { nullptr };
-        struct wl_listener destroyListener;
-
-        static void destroyNotify(struct wl_listener*, void*);
-    } m_client;
+    std::vector<uint32_t> m_bridgeIds;
 
     ClientBundle* m_clientBundle;
     struct wpe_view_backend* m_backend;
 
-    struct wl_list m_frameCallbacks;
-
     std::unique_ptr<FdoIPC::Connection> m_socket;
     int m_clientFd { -1 };
 };
