@@ -28,6 +28,7 @@
 #include "wpe-audio-server-protocol.h"
 #include "wpe-bridge-server-protocol.h"
 #include "wpe-video-plane-display-dmabuf-server-protocol.h"
+#include <algorithm>
 #include <cassert>
 #include <sys/socket.h>
 #include <unistd.h>
@@ -110,8 +111,11 @@ static const struct wl_surface_interface s_surfaceInterface = {
             return;
         }
 
-        wl_resource_set_implementation(callbackResource, nullptr, nullptr, nullptr);
-        surface.exportableClient->frameCallback(callbackResource);
+        wl_resource_set_implementation(callbackResource, nullptr, nullptr,
+            [](struct wl_resource* resource) {
+                wl_list_remove(wl_resource_get_link(resource));
+            });
+        surface.addFrameCallback(callbackResource);
     },
     // set_opaque_region
     [](struct wl_client*, struct wl_resource*, struct wl_resource*) { },
@@ -121,6 +125,7 @@ static const struct wl_surface_interface s_surfaceInterface = {
     [](struct wl_client*, struct wl_resource* surfaceResource)
     {
         auto& surface = *static_cast<Surface*>(wl_resource_get_user_data(surfaceResource));
+        surface.commit();
         WS::Instance::singleton().impl().surfaceCommit(surface);
     },
     // set_buffer_transform
@@ -144,13 +149,12 @@ static const struct wl_compositor_interface s_compositorInterface = {
             return;
         }
 
-        auto* surface = new Surface;
-        surface->client = client;
-        surface->id = id;
+        auto* surface = new Surface {surfaceResource};
         wl_resource_set_implementation(surfaceResource, &s_surfaceInterface, surface,
             [](struct wl_resource* resource)
             {
                 auto* surface = static_cast<Surface*>(wl_resource_get_user_data(resource));
+                WS::Instance::singleton().unregisterSurface(surface);
                 delete surface;
             });
     },
@@ -501,23 +505,48 @@ void Instance::releaseAudioPacketExport(struct wpe_audio_packet_export* packet_e
     wpe_audio_packet_export_send_release(packet_export->exportResource);
 }
 
-struct wl_client* Instance::registerViewBackend(uint32_t surfaceId, ExportableClient& exportableClient)
+void Instance::registerViewBackend(uint32_t bridgeId, ExportableClient& exportableClient)
 {
-    auto it = m_viewBackendMap.find(surfaceId);
+    auto it = m_viewBackendMap.find(bridgeId);
     if (it == m_viewBackendMap.end())
-        g_error("Instance::registerViewBackend(): " "Cannot find surface %" PRIu32 " in view backend map.", surfaceId);
+        g_error("Instance::registerViewBackend(): " "Cannot find surface with bridgeId %" PRIu32 " in view backend map.", bridgeId);
 
     it->second->exportableClient = &exportableClient;
-    return it->second->client;
 }
 
-void Instance::unregisterViewBackend(uint32_t surfaceId)
+void Instance::unregisterViewBackend(uint32_t bridgeId)
 {
-    auto it = m_viewBackendMap.find(surfaceId);
+    auto it = m_viewBackendMap.find(bridgeId);
     if (it != m_viewBackendMap.end()) {
         it->second->exportableClient = nullptr;
         m_viewBackendMap.erase(it);
     }
 }
 
+void Instance::unregisterSurface(Surface* surface)
+{
+    auto it = std::find_if(m_viewBackendMap.begin(), m_viewBackendMap.end(),
+        [surface](const std::pair<uint32_t, Surface*>& value) -> bool {
+            return value.second == surface;
+        });
+    if (it != m_viewBackendMap.end()) {
+        m_viewBackendMap.erase(it);
+        if (surface->exportableClient)
+            surface->exportableClient->bridgeConnectionLost(it->first);
+    }
+}
+
+void Instance::dispatchFrameCallbacks(uint32_t bridgeId)
+{
+    auto it = m_viewBackendMap.find(bridgeId);
+    if (it == m_viewBackendMap.end()) {
+        g_warning("Instance::dispatchFrameCallbacks(): "
+                  "Cannot find surface with bridgeId %" PRIu32 " in view "
+                  "backend map. Probably the associated surface is gone "
+                  "due to a premature exit in the client side", bridgeId);
+    } else {
+        it->second->dispatchFrameCallbacks();
+    }
+}
+
 } // namespace WS
